// Copyright (C) - lucien.povy@free.fr. Carantec 2014-2016. 2024
// This file must be used under the terms of the GNU GPL v2.0 licence.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// https://www.gnu.org/licenses/licenses.en.html
// ====================================================================
//This program gives partial fraction decomposition of a system or a rational.
// ====================================================================
function [resultat] = frac_decomp(r,flag)
//  ******************Input tests.
rhs = argn(2) ;
select rhs
  case 1 then
    flag = "no" ;
  case 2 then
    if type(flag) == 10  then
      if part(flag , 1) <> "y"  then
        flag= "no" ;
      end
    else
      flag = "no" ;
    end
end
if argn(2) > 2 then 
  error(msprintf(_("%s: Wrong number of input arguments :%d to %d: expected.\n"), "frac_decomp", 1,2))
end
if and(typeof(r) <> ["rational","zpk","state-space"]) | size(r,"*") > 1 then
  error(msprintf(_("%s: Wrong type for argument #%d : Rational or SISO linear system expected.\n"), "frac_decomp", 1))
end
if typeof(r) == "state-space" then 
  r = clean(ss2tf(r));
end
if typeof(r) == "zpk" then 
  r = zpk2tf(r);
end

//  ******************End of tests.
//  *****************Main program mroots.sci is used.
Num = r.num ; Den = r.den ;
x = poly(0,varn(Den)) ;
[rest ,elm1] = pdiv(Num , Den) ; 
G = rest/Den ;
rm = mroots(Den) ;// Mandatory.
rac = clean(rm(: ,1),0,1.e-14); mul = rm(: ,2) ;//roots and multiplicities.
//  *****************Real roots and multiplicities.
racr = rac(imag(rac) == 0); mulr = mul(imag(rac) == 0);
if racr == [] then
  polr = []; termr = [];
else
//  *****************Elementary polynomials with real roots.
  polr = [] ;
  for j = 1:length(racr)
    racrj = racr(j); mulrj = mulr(j) ; 
    polrj = (x - racrj)^(1 : mulrj)' ;
    polr = [polr ; polrj] ;
  end
  termr=(Den./polr).num ;
end
//  *****************Elementary polynomials with complex roots.
raci = rac(imag(rac) > 0) ; muli = mul(imag(rac) > 0) ;
if raci == [] then
  poli = [] ; termi = [] ;
else
  poli = [] ;termi = [] ;
  for j = 1:length(raci)
    racij = raci(j) ; mulij = muli(j) ;
    trinoj = real((x - racij)*(x - conj(racij))) ;
    TRINOj = [] ;
    for k = 1:mulij 
      TRINOj = [TRINOj,trinoj^k] ;
    end
    RESULTj = [Den ; x*Den]./.TRINOj ;
    termij = matrix(RESULTj , 2*mulij , 1).num ;
    termi = [termi;termij] ;
    polij = trinoj.^(1:mulij)';
 //imaginary elements
   POLIJ = [polij,polij/x]';
   Rij = matrix(POLIJ,2*mulij,1) ;
   poli = [poli ; Rij] ;
  end 
end
term = [termr;termi] ; pol = [polr;poli] ;
//  *****************The matrix.
A = coeff(term ,0:degree(Den) - 1)';//The matrix.
coerest = coeff(rest ,0:degree(Den) - 1)';
Alpha = A\coerest ;
elm = Alpha.*(ones(pol)./pol) ;
result = [elm1,elm'] ;
if part(flag , 1) <> "y"  then
  resultat = result ;
//Présentation. 
else
  indi = sum(mul(imag(rac) == 0)) +1 ;
  klim = sum(mul(imag(rac) > 0)) ;
  resultat = [result(1,1:indi)] ;
  for k = 1:2:2*klim
    resultat = [resultat,result(1,indi+k)+result(1,indi+k+1)] ;
  end
end
endfunction













